function simul = fixk_qe_dgp(kn,nsparse,beta,trunc)

deltan = 1/390/60; % 1 second Euler scheme
rho = -0.7; % leverage effect
nobs = nsparse * kn; % number of 1-sec returns

dL1 = stabletrunc(beta,trunc,nobs) * deltan^(1/beta); % Levy shock in vol factor 1
dL2 = stabletrunc(beta,trunc,nobs) * deltan^(1/beta); % Levy shock in vol factor 2
dW  = randn(nobs,1) * sqrt(deltan);

% change this if volatility jumps are needed
voljump = zeros(nobs,1);
    
% Initializing
V1 = zeros(nobs,1); V1(1) = 0.5;
V2 = zeros(nobs,1); V2(1) = 0.5;

% Diagonostic flags
err_count = 0; % Unsuccessful trials (i.e., negative variance)
finish_flag = 0; % Successful simulation

% Simulation via Euler
while ~finish_flag
    err_flag = 0;
    for i = 2:nobs
        V1(i) = V1(i-1) + 0.0128 * (0.4068 - V1(i-1)) * deltan ...
                        + 0.0954 * sqrt(V1(i-1)) * (rho * dW(i-1) + sqrt(1-rho^2) * dL1(i-1)) + voljump(i);
        V2(i) = V2(i-1) + 0.6930 * (0.4068 - V2(i-1)) * deltan ...
                        + 0.7023 * sqrt(V2(i-1)) * (rho * dW(i-1) + sqrt(1-rho^2) * dL2(i-1)) + voljump(i); 
        if V1(i) < 0 || V2(i) < 0
            % if get negative variance, flag err, redraw
            err_flag = 1;
            err_count = err_count + 1;
            dL1 = randn(nobs,1) * sqrt(deltan);
            dL2 = randn(nobs,1) * sqrt(deltan);
            dW  = randn(nobs,1) * sqrt(deltan);        
            break;
        end
    end
    if err_count > 100
        error('DGP does not really work!') % Give up
    end
    if ~err_flag
        % if loop finishes without err, finish, otherwise rerun
        finish_flag = 1;
    end
end

dxall = sqrt(V1 + V2) .* dW;
dx = sum(reshape(dxall,nsparse,kn),1)';

% Other things
simul.dx = dx;
simul.kn = kn;
simul.V1 = V1;
simul.V2 = V2;
simul.V = V1+V2;


% helper function
function x = stabletrunc(beta,trunc,n)
% simulate truncated beta-stable random variables
x = random('Stable',beta,0,1/sqrt(2),0,n,1);
    % simulate stable variable, setting the scale to 1/sqrt(2) to recover
    % Brownian motion when beta = 2
while sum(abs(x) > trunc) > 0
    % redraw the variables greater than truncation level
    tmpindex = (abs(x)>trunc);
    tmpn = sum(tmpindex);
    x(tmpindex) = random('Stable',beta,0,1/sqrt(2),0,tmpn,1);
end