%%
K = length(klist); kmax = max(klist);
deltan = 1/23400 * nsparse;
chat = zeros(MC,K);
ctrue = zeros(MC,K); 
boot_cv_var = zeros(MC,K);
boot_cv_log = zeros(MC,K);
tic;
parfor mc = 1:MC
    simul = fixk_qe_dgp(kmax,nsparse,beta,trunc);
    V = simul.V;
    dx_max = simul.dx;
    tmp_chat = zeros(1,K);
    tmp_ctrue = zeros(1,K);
    tmp_boot_cv_var = zeros(1,K);
    tmp_boot_cv_log = zeros(1,K);
    for kloop = 1:K
        k = klist(kloop);
        dx = dx_max(1:k);
        tmp_chat(kloop) = mean(dx.^2)/deltan;
        tmp_ctrue(kloop) = V(k*nsparse);
        % bootstrap
        chat_boot = zeros(nboot,1);
        for i = 1:nboot
            index_boot = unidrnd(k,k,1);
            dx_boot = dx(index_boot);
            chat_boot(i) = mean(dx_boot.^2)/deltan;
        end
        % bootstrap t-stat for variance
        tstat_var_boot = sqrt(k) * (chat_boot - tmp_chat(kloop)) ...
                               ./ sqrt(2) ./ chat_boot;
        tmp_boot_cv_var(kloop) = prctile(abs(tstat_var_boot),90);              
        % bootstrap t-stat for log variance
        tstat_log_boot = sqrt(k/2) * (log(chat_boot) - log(tmp_chat(kloop)));
        tmp_boot_cv_log(kloop) = prctile(abs(tstat_log_boot),90);              
    end
    chat(mc,:) = tmp_chat;
    ctrue(mc,:) = tmp_ctrue;
    boot_cv_var(mc,:) = tmp_boot_cv_var;
    boot_cv_log(mc,:) = tmp_boot_cv_log;
end
toc;
%% Compute rejection rate

% The inverse statistic (associated with inverse chi2 HD critical values)
T_fixk = ctrue./chat;

% Conventional tstat for variance
T_gauss = repmat(sqrt(klist),MC,1) .* (chat - ctrue) ./ sqrt(2) ./ chat;

% Fixed-k stat for log var
T_logfixk = log(chat) - log(ctrue);

% Conventional tstat for log variance
T_loggauss = repmat(sqrt(klist),MC,1) .* (log(chat) - log(ctrue)) ./ sqrt(2);

% Load pre-computed fixed-k critical values
load('HDI90_InverseChi2'); Inverse90 = result; % 90 HD intervals for inv-chi2 
load('SYM90_LogScaledChi2');  LogSYM90 = result; % crit val for sym log CIs

p_fixk = zeros(K,1);
p_gauss = zeros(K,1);
p_boot = zeros(K,1);
p_logfixk = zeros(K,1);
p_loggauss = zeros(K,1);
p_logboot = zeros(K,1);
for i = 1:K
    k = klist(i);
    
    % Variance
    
    % fixed k  
    p_fixk(i) = fixedk_coverage(T_fixk(:,i),k,Inverse90);
   
    % gaussian
    p_gauss(i) = mean(abs(T_gauss(:,i))<=norminv(0.95));
    
    % boot
    p_boot(i) = mean(abs(T_gauss(:,i))<=boot_cv_var(:,i));

    % Log variance
    
    % fixed k
    p_logfixk(i) = fixedk_coverage(T_logfixk(:,i),k,LogSYM90);   
    
    % gaussian
    p_loggauss(i) = mean(abs(T_loggauss(:,i))<=norminv(0.95));       
    
    % boot
    p_logboot(i) = mean(abs(T_loggauss(:,i))<=boot_cv_log(:,i));

end



filename = sprintf('simulation_result_qe_%s',msg);
save(['./mat/' filename],'p_fixk','p_gauss','p_boot',...
                         'p_logfixk','p_loggauss','p_logboot',...
                         'klist','nsparse');



