%addpath(genpath('../library/'));
MC = 100000; nsparse = 60;
nblist = [1 2 3 6 39 39*5 39*10]; 

K = length(nblist); maxnb = max(nblist);
k = 10;
deltan = 1/23400 * nsparse; 

supstat_block = zeros(MC,K);
supstat_1min = zeros(MC,K);
supstat_1sec = zeros(MC,K);
    % max |log(chat_t) - log(c_t)|, t in every block, every min, every sec
tic
parfor mc = 1:MC
    simul = fixedk_dgp(maxnb * k,nsparse);
    dx = simul.dx;
    dx_block = reshape(dx,k,maxnb);
    chat_maxnb = mean(dx_block.^2,1)/deltan;
        % row vector of chat for all blocks
    logV = log(simul.V);
        % V at 1 second
    logV1 = logV(1:nsparse:end);
        % V at 1 minute
    logVk = logV(1:(k*nsparse):end);
        % V at each block

    tmp_sup_block = zeros(1,K);
    tmp_sup_1min = zeros(1,K);
    tmp_sup_1sec = zeros(1,K);
    
    for nbloop = 1:K
        nb = nblist(nbloop);
        logchat = log(chat_maxnb(1:nb));
            % log(chat) for the first nb blocks
        tmp_sup_block(nbloop) = max(abs(logchat' - logVk(1:nb)));
            % sup stat for blocks
        
        logchat1min = repmat(logchat,k,1);
        tmp_sup_1min(nbloop) = max(abs(logchat1min(:) - logV1(1:(nb*k))));
        
        logchat1sec = repmat(logchat,k*nsparse,1);
        tmp_sup_1sec(nbloop) = max(abs(logchat1sec(:) - logV(1:(nb*k*nsparse))));
    end
    supstat_block(mc,:) = tmp_sup_block;
    supstat_1min(mc,:) = tmp_sup_1min;
    supstat_1sec(mc,:) = tmp_sup_1sec;
end
toc
%% Compute rejection rate, save result
rej90 = zeros(K,3);
rej95 = zeros(K,3);
for nbloop = 1:K
    nb = nblist(nbloop);
    cv90 = Symmetric_LogScaleChi2(k,0.9^(1/nb));
        % Symmetric_LogScaleChi2(k,q) compute q-quantile of
        % |log(scaled chi2 df = k)|
    cv95 = Symmetric_LogScaleChi2(k,0.95^(1/nb));
    rej90(nbloop,:) = [mean(supstat_block(:,nbloop)<=cv90),...
                     mean(supstat_1min(:,nbloop)<=cv90),...
                     mean(supstat_1sec(:,nbloop)<=cv90)];
    rej95(nbloop,:) = [mean(supstat_block(:,nbloop)<=cv95),...
                     mean(supstat_1min(:,nbloop)<=cv95),...
                     mean(supstat_1sec(:,nbloop)<=cv95)];
end
% rej90
% rej95

filename = ['simulation_result_uniform_nsparse' num2str(nsparse)];
save(['./mat/' filename],'rej90','rej95','nblist','nsparse');

